#include "global.h"

/* Routine for returning the Nash Equilibrium as a type and as with values */

int NE_function(double* pr_NE,double* eps_r,double** p_mat)
{
  double a,b1,b2,c1,c2,d,e1,e2,qt,pt;
  int inc,output;
  double NE_mat[4];
  double M1,M2,m1,m2;

  a  = p_mat[0][0];
  b1 = p_mat[1][0];
  b2 = p_mat[1][1];
  c1 = p_mat[2][0];
  c2 = p_mat[2][1];
  d  = p_mat[3][0];

  e1 = eps_r[0];
  e2 = eps_r[1];

  /* Initialize NE_mat */
  for(inc=0; inc<4; inc++) {
    NE_mat[inc] = 0.0;
  }

  if ((a-c1>=b1-d) && (a-b2>=c2-d)) {
    /* Case 1 of NE */
    m1 = b1 - d;
    m2 = c2 - d;
    M1 = a  - c1;
    M2 = a  - b2;
	
    if ((((e1>m1) && (e1<M1)) && ((e2>m2) && (e2<M2)) && (M1>m1)) && (M2>m2)) {
      pt=(e1-(b1-d))/((a-c1)-(b1-d));
      qt=(e2-(c2-d))/((a-b2)-(c2-d));
      output = 6;
		
      NE_mat[0]= qt; NE_mat[1] = 1.0 - qt; NE_mat[2] = pt; NE_mat[3] = 1.0 - pt;
				    
    }
    else if ((e1>m1) && (e2>m2)) {
      output = 4;
    }
    else if ((e1<m1) && (e2>M2)) {
      output = 2;
    }
    else if ((e1>M1) && (e2<m2)) {
      output = 3;
    }
    else {
      output = 1;
    }
  }
  else if ((a-c1<=b1-d) && (a-b2<=c2-d)) {
    /* Case 2 of NE */
    M1 = b1 - d;
    M2 = c2 - d;
    m1 = a  - c1;
    m2 = a  - b2;
	
    if ((((e1>m1) && (e1<M1)) && ((e2>m2) && (e2<M2)) && (M1>m1)) && (M2>m2)) {
      pt=(e1-(b1-d))/((a-c1)-(b1-d));
      qt=(e2-(c2-d))/((a-b2)-(c2-d));
      output = 7;
		
      NE_mat[0]= qt; NE_mat[1] = 1.0 - qt; NE_mat[2] = pt; NE_mat[3] = 1.0 - pt;

    }
    else if ((e1>M1) && (e2>M2)) {
      output = 4;
    }
    else if ((e1<M1) && (e2>m2)) {
      output = 2;
    }
    else if ((e1>m1) && (e2<M2)) {
      output = 3;
    }
    else {
      output = 1; 
    }
  }
  else if ((a-c1<=b1-d) && (a-b2>=c2-d)) {
    /* Case 3 of NE */
    M1 = b1 - d;
    m2 = c2 - d;
    m1 = a  - c1;
    M2 = a  - b2;
	
    if ((((e1>m1) && (e1<M1)) && ((e2>m2) && (e2<M2)) && (M1>m1)) && (M2>m2)) {
      pt=(e1-(b1-d))/((a-c1)-(b1-d));
      qt=(e2-(c2-d))/((a-b2)-(c2-d));
      output = 5;
		
      NE_mat[0]= qt; NE_mat[1]= 1.0 - qt; NE_mat[2]= pt; NE_mat[3] = 1.0 - pt;

    }
    else if ((e1>M1) && (e2>m2)) {
      output = 4;
    }
    else if ((e1<M1) && (e2>M2)) {
      output = 2;
    }
    else if ((e1>m1) && (e2<m2)) {
      output = 3;
    }
    else {
      output = 1;
    }
  }
  else {
    /* Case 4 of NE */		
    m1 = b1 - d;
    m2 = c2 - d;
    M1 = a  - c1;
    M2 = a  - b2;
	
    if ((((e1>m1) && (e1<M1)) && ((e2>m2) && (e2<M2)) && (M1>m1)) && (M2>m2)) {
      pt=(e1-(b1-d))/((a-c1)-(b1-d));
      qt=(e2-(c2-d))/((a-b2)-(c2-d));
      output = 5;
		
      NE_mat[0] = qt; NE_mat[1] = 1.0 - qt; NE_mat[2] = pt; NE_mat[3] = 1.0 - pt;

    }
    else if ((e1<m1) && (e2>m2)) {
      output = 4;
    }
    else if ((e1<m1) && (e2>m2)) {
      output = 2;
    }
    else if ((e1>M1) && (e2<M2)) {
      output = 3;
    }
    else {
      output = 1; 
    }
  }


  /* Return the probabilities that will be useful, along with type_NE, in computing L */
 
  pr_NE[0] = NE_mat[0]*NE_mat[2];
  pr_NE[1] = NE_mat[0]*NE_mat[3];
  pr_NE[2] = NE_mat[1]*NE_mat[2];
  pr_NE[3] = NE_mat[1]*NE_mat[3];
  return output;
}
