#include "global.h"

const int obs = 1212;	
const int dim_x = 5;
const int R = 1000;
const double TOL = 0.05;
const int tolerance = 0;
const int dim_theta = 12;
const int mrkt = 32;
const int Nsave = 1;

int main(int argc,char** argv)
{
  int m,i,j,inc,jnc,D1,D2,M,fail_m,result,length_result = 0;
  unsigned long s = (unsigned long) time(NULL);
  initialize_random_generator(s);

  unsigned long int lim_theta;
  
  /*timing*/

  time_t start,end;
  double dif;
  time (&start);
  /*model restrictions*/

  double t,Fbest;

  double L[16];
  int* Index_result;

  double** Theta_mat;
  double** xplic_mat;
  double** phat_mat;
  double** ptr;
  Theta theta;
  Xplic xplic;

  phat_mat = malloc(mrkt*sizeof(double));
  for(i=0; i<mrkt; i++) {
    phat_mat[i] = malloc(16*sizeof(double));
  }
  result = Read_File(phat_mat,"proba.bin",mrkt,16);
  if (result == 0) printf("imported proba\n");

  xplic_mat = malloc(mrkt*sizeof(double));
  for(i=0; i<mrkt; i++) {
    xplic_mat[i] = malloc(dim_x*sizeof(double));
  }

  result = Read_File(xplic_mat,"xpli.bin",mrkt,dim_x);
  if (result==0) printf("imported xplic\n");

  /*Theta_mat = malloc(lim_theta*sizeof(double));
  for(i=0; i<lim_theta; i++) {
    Theta_mat[i] = malloc(dim_theta*sizeof(double));
  }

  result = Read_File(Theta_mat,"theta_mat.bin",lim_theta,dim_theta);
  if (result==0) printf("imported theta\n");*/

  lim_theta = gen_theta(ptr,0);

  Theta_mat = malloc(lim_theta*sizeof(double));
  for(i=0; i<lim_theta; i++) {
   Theta_mat[i] = malloc(dim_theta*sizeof(double));
  }

  lim_theta = gen_theta(Theta_mat,1);
  
  Index_result = malloc(lim_theta*sizeof(int));
  for(inc=0; inc<lim_theta; inc++) {
    Index_result[inc] = 0;
  }
  
  M = (int)lim_theta/Nsave;
  
  
for (jnc=0;jnc<Nsave;jnc++){
D1 = jnc*M;
D2 = (jnc+1)*M;
  /* -------------Open MP section------------------------ */
#pragma omp parallel for default(shared) private(inc,fail_m,theta,xplic,i,m,L,Fbest) reduction(+:length_result)
  for(inc=D1; inc<D2; inc++) {
    fail_m = 0;
	
    theta.bh0     = Theta_mat[inc][0];
    theta.bn0     = Theta_mat[inc][1];
    theta.padl    = Theta_mat[inc][2];
    theta.pmarry  = Theta_mat[inc][3];
    theta.alpha   = Theta_mat[inc][4];
    theta.cbeta   = Theta_mat[inc][5];
    theta.csex    = Theta_mat[inc][6];
    theta.cdist   = Theta_mat[inc][7];
    theta.mu      = Theta_mat[inc][8];
    theta.sigma   = Theta_mat[inc][9];
    theta.var_xsi = Theta_mat[inc][10];
    theta.m_er    = Theta_mat[inc][11];
	
    m = 0;
    do {
      /*xplic.pADL   = xplic_mat[m][0];
      xplic.pMARRY = xplic_mat[m][1];
      xplic.c1SEX  = xplic_mat[m][2];
      xplic.c1DIST = xplic_mat[m][3];
      xplic.c2SEX  = xplic_mat[m][4];
      xplic.c2DIST = xplic_mat[m][5];
      
      
      Reduced formulation where Prim x sex is considered in xpli*/
      
      xplic.pADL   = xplic_mat[m][0];
      xplic.pMARRY = xplic_mat[m][1];
      xplic.c1DIST = xplic_mat[m][2];
      xplic.c2DIST = xplic_mat[m][3];
      xplic.c1SEX  = xplic_mat[m][4];
      xplic.c2SEX  = 0.0;

      
      /* Simulate L */
      L_function(L,&theta,&xplic);
				
      Fbest = phat_mat[m][0] - L[0];
      for(i=1; i<16; i++) {
	t = phat_mat[m][i] - L[i];
	if (t > Fbest) Fbest = t;
      }
		
      m++;
      if (Fbest > TOL) fail_m++;
				      
    } while(m < mrkt && fail_m <= tolerance);
	
    if (fail_m <= tolerance) {
      Index_result[inc] = 1;
      length_result++;
    }
  }
  /*-----------------End Open MP section-------------------*/

  printf("number result:%d\n",length_result);

  if (length_result > 0) {
    double** Theta_result;
    Theta_result = malloc(length_result*sizeof(double));
    for(i=0; i<length_result; i++) {
      Theta_result[i] = malloc(dim_theta*sizeof(double));
    }

    i = 0;
    for(inc=0; inc<D2; inc++) {
      if(Index_result[inc] == 1){
	for(j=0; j<dim_theta; j++) {
	  Theta_result[i][j] = Theta_mat[inc][j];
	}
      i++;
      }
    }

    result = Write_File(Theta_result,"Theta_result.bin",length_result,dim_theta);
    if (result == 0) printf("Yeah!!! - Exported theta_result.bin\n");

    for(i=0; i<length_result; i++) {
      free(Theta_result[i]);
    }
    free(Theta_result);
  }
  }
  /* Free the memory*/

  free(Index_result);

  for(i=0; i<mrkt; i++) {
    free(phat_mat[i]);
  }
  free(phat_mat);

  for(i=0; i<mrkt; i++) {
    free(xplic_mat[i]);
  }
  free(xplic_mat);

  for(i=0; i<lim_theta; i++) {
    free(Theta_mat[i]);
  }
  free(Theta_mat);

  finalize_random_generator();

  time(&end);

  dif = difftime(end,start);
  printf("Elapsed time: %.2f\n", dif);
  return 0;
}


/*----------------------------------------------------------------------*/
/*  Level 1      */
/*----------------------------------------------------------------------*/


/*----------------*/
/* Function LB_inc*/
/*----------------*/

void L_function(double* Lv, Theta* theta,Xplic* xplic)
{
  /* Valuation and profit parameters*/
  double** Vi;
  double** CVi;
  double** profi1;
  int    type_NE,choice1[4];
  int    i,inc,jnc,r;
  double t,/*er1,er2,*/LB_bis[6],P_er[3],prNE[4],epsi_r[2],Lb[96];
  const double csex = theta->csex;

  /*Random draw parameters*/
  double epsi1[R],epsi2[R],xsi1[R],xsi2[R],xsi3[R],xsi4[R];

  /*double** M_er1;
  double** M_er2;*/

  Vi = malloc(3*sizeof(double));
  CVi = malloc(3*sizeof(double));
  for(i=0; i<3; i++) {
    Vi[i] = malloc(4*sizeof(double));
    CVi[i] = malloc(4*sizeof(double));
  }

  profi1 = malloc(4*sizeof(double));
  for(i=0; i<4; ++i) {
    profi1[i] = malloc(2*sizeof(double));
  }

  /*M_er1 = malloc(R*sizeof(double));
  M_er2 = malloc(R*sizeof(double));
  for(i=0; i<R; i++) {
    M_er1[i] = malloc(3*sizeof(double));
    M_er2[i] = malloc(3*sizeof(double));
  }
 
  /*-------------Calculation of the valuation-----------*/
  V_function(theta,xplic,Vi);

  /* ------Random draws and insertion in the Valuation------*/
  generate_gaussian_vector(epsi1,R,theta->mu,theta->sigma);
  generate_gaussian_vector(epsi2,R,theta->mu,theta->sigma);
  generate_gaussian_vector(xsi1,R,0,theta->var_xsi);
  generate_gaussian_vector(xsi2,R,0,theta->var_xsi);
  generate_gaussian_vector(xsi3,R,0,theta->var_xsi);
  generate_gaussian_vector(xsi4,R,0,theta->var_xsi);

  /*P_er[0] = theta->m_er;
  P_er[1] = theta->m_er;
  P_er[2] = 1.0-2.0*theta->m_er;

  generate_multinomial_vector(M_er1,R,3,P_er);
  generate_multinomial_vector(M_er2,R,3,P_er);*/

  /* --------Calculation of the NE from profit and epsi-------
     ----------------and getting the value of L*-----------------*/

  for(inc=0; inc<16; inc++) {
    Lv[inc] = 0.0;
  }

  for(r=0; r<R; r++) {
    for(inc=0; inc<3; inc++) {
      for(jnc=0; jnc<4; jnc++) {
	CVi[inc][jnc] = Vi[inc][jnc];
      }
    }
	
    /*er1 = csex*(M_er1[r][0] - M_er1[r][1]);
    er2 = csex*(M_er2[r][0] - M_er2[r][1]);*/
	
    CVi[0][0] += xsi1[r];
    CVi[1][0] += xsi1[r]; 
    CVi[2][0] += xsi1[r];
	
    CVi[0][1] += xsi2[r];
    CVi[1][1] += xsi2[r];
    CVi[2][1] += xsi2[r];
	
    CVi[0][2] += xsi3[r];/*(xsi3[r] + er1);*/
    CVi[1][2] += xsi3[r];
    CVi[2][2] += xsi3[r];
	
    CVi[0][3] += xsi4[r];/*(xsi4[r] + er2);*/
    CVi[1][3] += xsi4[r];
    CVi[2][3] += xsi4[r];
	
    /*--------------------To compute profit-------------------*/

    profit(CVi,profi1,choice1);
	
    epsi_r[0] = epsi1[r];
    epsi_r[1] = epsi2[r];
	
    type_NE = NE_function(prNE,epsi_r,profi1);

    /* Initialize Lb */	
    for(inc=0; inc<96; inc++) {
      Lb[inc] = 0.0;
    }
		
    LB_inc(Lb,prNE,type_NE,choice1);
	
    for(inc=0; inc<16; inc++) {
      t = Lb[6*inc];
      for(jnc=1; jnc<6; jnc++) {			
	if (Lb[6*inc+jnc] > t) t = Lb[6*inc+jnc];
      }
      Lv[inc] += (t/R);
    }
  }

  for(i=0; i<3; i++) {
    free(Vi[i]);
    free(CVi[i]);
  }
  free(Vi);
  free(CVi);
 
  for(i=0; i<4; i++) {
    free(profi1[i]);
  }
  free(profi1);

 /* for(i=0; i<R; i++) {
    free(M_er1[i]);
    free(M_er2[i]);
  }
  free(M_er1);
  free(M_er2);*/
}


/*----------------------------------------------------------------------*/
/*  Level 2       */
/*----------------------------------------------------------------------*/


/*----------------*/
/* Function LB_inc*/
/*----------------*/

void LB_inc(double* LB,double* pr_NE,int ty,int* c)
{
  int i,j;
  double tp_L[16];

  /* LB for the pure str. equilibrium */
  if ((ty <= 4) && (ty > 0)) {		
    Li(tp_L,1,c[ty-1]);
    for(i=0; i<16; i++) {
      LB[6*i] = tp_L[i];
    }
  }
  else {
    /* (ty>4) LB for the mixed str. equilibrium */
    for(j=0; j<4; j++) {
      Li(tp_L,pr_NE[j],c[j]);
      for(i=0; i<16; i++) {
	LB[6*i+j+2] = tp_L[i];
      }
    }
    if (ty == 5) {
			
    }
    else if (ty == 6) {
      Li(tp_L,1,c[3]);
      for(i=0; i<16; i++) {
	LB[6*i] = tp_L[i];
      }
      Li(tp_L,1,c[0]);
      for(i=0; i<16; i++) {
	LB[6*i+1] = tp_L[i];
      }			
    }
    else if (ty == 7) {
      Li(tp_L,1,c[1]);
      for(i=0; i<16; i++) {
	LB[6*i] = tp_L[i];
      }
      Li(tp_L,1,c[2]);
      for (i=0; i<16; i++) {
	LB[6*i+1] = tp_L[i];
      }
    }
    else {
      printf("Error with the type\n");
    }
  }	
}

/*----------------------------------------------------------------------*/
/*  Level 3      */
/*----------------------------------------------------------------------*/


/*----------------*/
/* Function Li    */
/*----------------*/
		
void Li(double* L,double p,int ch)
{
  int inc;
  int row[7];

  L[15] = 1.0;
  for(inc=0; inc<15; inc++) {
    L[inc] = 0.0;
  }

  switch(ch) {
  case 0: 
    row[0] = 1; row[1] = 5;
    row[2] = 6; row[3] = 7;
    row[4] = 11; row[5] = 12;
    row[6] = 13;
    break; 
		
  case 1: 
    row[0] = 2; row[1] = 5;
    row[2] = 8; row[3] = 9;
    row[4] = 11; row[5] = 12;
    row[6] = 14;
    break;

  case 2: 
    row[0] = 3; row[1] = 6;
    row[2] = 8; row[3] = 10;
    row[4] = 11; row[5] = 13;
    row[6] = 14;
    break;

  case 3:	
    row[0] = 4; row[1] = 7;
    row[2] = 9; row[3] = 10;
    row[4] = 12; row[5] = 13;
    row[6] = 14;
    break;
  }

  for(inc=0; inc<7; inc++) {
    L[row[inc]] = p;
  }
}

