function [Xplic, X]=get_x_y(A,restr,nb)

% This routine returns the tractable variables definitions for our
% estimation. It takes the values from the matrix of data (possibly
% simulated A_sim.mat) and returns matrix of given variables.

% A will be constructed as following:
%      Parents            child i                         additional      family size
% [Y  ADL p_marry  sex child c_marry dist1 dist2 ... add_X0 add_ci add_cj    N   ]
dim_A=length(A(1,:));

%% Definition of the variable Xplic

%-----------------------------------------------------------------  
% Care options
%-----------------------------------------------------------------  
Xplic.Y  = A(:,1);
%*** Think of a restriction to merge "Home" and "Nursing"

%-----------------------------------------------------------------  
% Parents
%-----------------------------------------------------------------  
Xplic.ADL    = A(:,2)* restr.ADL;
Xplic.p_marry= A(:,3)* restr.p_marry;

%-----------------------------------------------------------------  
% Children characteristics
%-----------------------------------------------------------------  
l=3;
l_count=1;
while l_count<=nb.max_player

    Xplic.X(l_count).sex    =A(:,l+1)* restr.sex;
    Xplic.X(l_count).child  =A(:,l+2)* restr.child;
    Xplic.X(l_count).c_marry=A(:,l+3)* restr.c_marry;
    Xplic.X(l_count).dist1  =A(:,l+4)* restr.dist1;
    Xplic.X(l_count).dist2  =A(:,l+5)* restr.dist2;
    l=l+5;
    l_count=l_count+1;

end
%-----------------------------------------------------------------  
% Additional characteristics
%-----------------------------------------------------------------  
l_end=l;

% Parents

if nb.add_p>0
    l_start= l_end+1;
    l_end  = l_end+nb.size_add_p;
    x_temp = A(:,l_start:l_end);
    Xplic.add_X0 = x_temp(:,restr.p1==1);
end

% Children
if nb.add_c>0
    l_count=1;
    while l_count<=nb.max_player
        l_start=l_end+1;
        l_end=l_end+nb.size_add_c;
        x_temp =A(:,l_start:l_end);
        Xplic.X(l_count).cj=x_temp(:,restr.c1==1);
        l_count=l_count+1;
    end
end

% Family size
Xplic.N = A(:,dim_A);

%% Definition of the variable X

% The objective is to remove the variable that are not needed because of
% the restriction.

% Vector of restriction
v_restr_p = [restr.ADL restr.p_marry];
v_restr_c = horzcat(restr.sex, restr.child , restr.c_marry, restr.dist1 , restr.dist2);
v_restr_c = repmat(v_restr_c,1,nb.max_player);
v_restr_c1 = repmat(restr.c1,1,nb.max_player);

% Alternative definition
% v_restr_p1 = ones(1,nb.add_p);
% v_restr_c1 = ones(1,nb.add_c);


v_restr=horzcat(v_restr_p,v_restr_c,restr.p1,v_restr_c1,restr.N);

X= A(:,2:dim_A);

% Implement the restrictions to exclude the useless variables.
X= X(:,v_restr==1);

%% Comments and additional notes

% for testing matters
% A_data=[round(4*rand(1000,1)) round(rand(1000,21)) round(2*rand(1000,1))];
% 
% savefile='A_data.mat'
% save(savefile,'A_data');

