function [restr,nb,p_name]=spec_rest(nb)

% This routine allows the user to specify the variables to include in the
% formulation of the valuation function.
% Value "1" for a restriction means the variable is included.
% Value "0" for a restriction means the variable is excluded.

%% Restriction of specification

%-------------------------------------------------------
% Parents characteristics
%-------------------------------------------------------
restr.ADL =1;
restr.p_marry = 1;

restr.ph_ADL    =1*restr.ADL;
restr.pn_ADL    =0*restr.ADL;
restr.ph_marry  =1*restr.p_marry;
restr.pn_marry  =0*restr.p_marry;
restr.bh0  =1;
restr.bn0  =1;


%-------------------------------------------------------
% Children characteristics
%-------------------------------------------------------
restr.sex    =0;
restr.child  =0;
restr.c_marry=0;
restr.dist1  =0;
restr.dist2  =1;
restr.prim   =1;
restr.beta   =1;

%-------------------------------------------------------
% Parameters of the errors terms
%-------------------------------------------------------
restr.mu = 1;
restr.sigma = 1;
restr.var_xsi = 1;


%-------------------------------------------------------
% Additional variables for parents
%-------------------------------------------------------
    % We can add in the specifiction additional variable about the parents,
    % e.g, sex, age, existence of a help program

%ADL1 and sex of first born
% To not take in account ADL1 p1(1)==0. To not take in acount prim_sex
% p1(2)==0.
nb.size_add_p=2; 
restr.p1 = [0 1]; % can be extended to more variables or taken empty
nb.add_p = sum(restr.p1);

%-------------------------------------------------------
% Additional variables for children
%-------------------------------------------------------
% We can add in the specifiction additional variable about the children,
% e.g, age, variable of differences, oldest male child
% Alternative definition
nb.size_add_c=0;
restr.c1 = []; % can be extended to more variables or taken empty
nb.add_c = sum(restr.c1);

%-------------------------------------------------------
% Other
%-------------------------------------------------------
restr.N        = 0; % Take in account different family size
nb.disability  = 0; % Model with different disabilities
nb.reduced     = 1; % Reduced model with prim_sex
nb.smooth      = 0; % Use a nonsmoothed nonparametric mean when = 0.
nb.multinomial = 0; % Use a multinomial logit. Set nb.smooth = 0.

%% Resultant parameters of the model

nb.char_p = restr.ADL+restr.p_marry+nb.add_p;
nb.char_c = restr.sex+restr.child+restr.c_marry+restr.dist1+restr.dist2+nb.add_c;

%% Restrictions on theta

V = vertcat(restr.bh0, restr.bn0,restr.ph_ADL,restr.pn_ADL, restr.ph_marry,restr.pn_marry,restr.sex,restr.child,restr.c_marry,restr.dist1,restr.dist2, restr.prim, restr.mu, restr.sigma, restr.var_xsi, restr.p1',restr.beta);
C = strvcat('bh0','bn0','ph_ADL','pn_ADL','ph_marry','pn_marry','sex','child','c_marry','dist1','dist2', 'alpha', 'mu', 'sigma', 'var_xsi','ph_ADL1','prim_sex','beta');
p_name.psave = C(V==1,:);
p_name.pxclu = C(V==0,:);
p_name.V     = V;

dim_theta = length(p_name.psave(:,1));

