#include "global.h"

const int obs = 4;	
const int dim_x = 2;
const int nbjeu = 5000;
const int dim_p_eg   = 128;
const int dim_pident = 128;
const int dim_eql_eg = 26;

const double TOL = 1e-10;
const int tolerance = 0;
const int dim_th = 3;

const int Nsave = 1;

int main(int argc,char** argv)
{
  int m,i,j,row,inc,jnc,fail_m,result,test_stop,length_result = 0,rejection=0;
  /*unsigned long s = (unsigned long) time(NULL);
  initialize_random_generator(s);*/

  unsigned long int lim_th;
  int* Index_result;
  
  /*Import the adjacency matrix*/
  double** adj_mat_eg;
  
   adj_mat_eg = malloc(dim_p_eg*sizeof(double));
  for(i=0; i<dim_p_eg; i++) {
    adj_mat_eg[i] = malloc(dim_eql_eg*sizeof(double));
  }
  result = Read_File(adj_mat_eg,"adj_mat_dec.bin",dim_p_eg,dim_eql_eg);
  if (result == 0) printf("imported adj_mat_eg\n");

  /*model restrictions*/

  double L,t,alpha,beta,c,th,g1,g2;

  double* Q; /*Receives the value of the capacity*/

  double** Theta_mat;
  double** xplic_mat;
  double** phat_mat_eg;
  double** pident_mat_eg;
  
  /*Importing matrix of probabilities*/    
  pident_mat_eg = malloc(obs*sizeof(double));
  for(i=0; i<obs; i++) {
    pident_mat_eg[i] = malloc(dim_pident*sizeof(double));
  }
  result = Read_File(pident_mat_eg,"pident_mat_eg.bin",obs,dim_pident);
  if (result == 0) printf("imported pident_mat_eg\n");
  
  row=(int)obs*nbjeu;
  phat_mat_eg = malloc(row*sizeof(double));
  for(i=0; i<row; i++) {
    phat_mat_eg[i] = malloc(dim_p_eg*sizeof(double));
  }
  result = Read_File(phat_mat_eg,"phat_mat_eg_90.bin",row,dim_p_eg);
  if (result == 0) printf("imported phat_mat_eg\n");
  
  /*Importing the covariates*/
  xplic_mat = malloc(obs*sizeof(double));
  for(i=0; i<obs; i++) {
    xplic_mat[i] = malloc(dim_x*sizeof(double));
  }
  if (dim_x==1){
  result = Read_File(xplic_mat,"Xplic_1.bin",obs,dim_x);
  }
  else {
  result = Read_File(xplic_mat,"Xplic_2.bin",obs,dim_x);
  }
  if (result==0) printf("imported xplic\n");
  
  /*Generate Theta to be tested*/
  
  lim_th = gen_theta_3(Theta_mat,0);

  Theta_mat = malloc(lim_th*sizeof(double));
  for(i=0; i<lim_th; i++) {
   Theta_mat[i] = malloc(dim_th*sizeof(double));
  }

  lim_th = gen_theta_3(Theta_mat,1);

  Index_result = malloc(lim_th*sizeof(int));
  for(inc=0; inc<lim_th; inc++) {
    Index_result[inc] = 0;
  }
  
  /*timing*/

  time_t start,end;
  double dif;
  time (&start);
    
 
  /* -------------Open MP section------------------------ */
  #pragma omp parallel for default(shared) private(inc,fail_m,i,j,m,L,t,test_stop,Q,alpha,beta,c,th,g1,g2) reduction(+:length_result)
  for(inc=0; inc<lim_th; inc++) {
  
    fail_m = 0;
    
    Q= malloc(dim_eql_eg*sizeof(double));
  	for(i=0; i<dim_eql_eg; i++) {
    	Q[i] = 0.0;
  	}  
	
    alpha = Theta_mat[inc][0];
    beta  = Theta_mat[inc][1];
    c     = alpha;
    th    = Theta_mat[inc][2];   
	
    m = 0;    
    test_stop = 0;
    
    do { /*market iteration*/
      if (((beta==0.0)||((beta >0.0)&&(2.0*c-3.0*c*(1.0+beta)>=-th))||((beta <0.0)&&(c-2.0*c*(1.0+beta)>=-th))) && (th >= alpha))
      { t=0.0;  
        g1  = xplic_mat[m][0];
        g2  = xplic_mat[m][1];
        t = Q_lkhd_eg(Q,alpha,th,beta*g1,beta*g2); /*If we introduce beta ~= 0 then it might needs to be in the loop for each market*/
        if (fabs(t-1.0)>TOL) printf("problem with p for beta %.2f\n",t);               
                
        i=0;
        do{	
          L=0;
          for (j=0; j<dim_eql_eg; j++){
	    L = L + adj_mat_eg[i][j]*Q[j];
	    /*printf("adj_mat_eg:%f,Q:%f\n",adj_mat_eg[i][j],Q[j]);*/
	    }
	    /*printf("L:%f,Pident_mat:%f\n",L,pident_mat_eg[m][i]);*/
	    if (L - pident_mat_eg[m][i] < - TOL) test_stop++;
	    i++;
        } while ((test_stop == 0) && (i < dim_p_eg));
      }
      else{
        test_stop++;
       } 
      if (test_stop > 0) fail_m++;
      m++;
      				      
    } while(m < obs && fail_m <= tolerance);
	
    if (fail_m <= tolerance) {     
      Index_result[inc] = 1;
      length_result++;
    }
    free(Q);
    
  }
  /*-----------------End Open MP section-------------------*/

 printf("number of points in identified region:%d\n",length_result);

 if (length_result > 0) {
    double** Theta_result;
    Theta_result = malloc(length_result*sizeof(double));
    for(i=0; i<length_result; i++) {
      Theta_result[i] = malloc(dim_th*sizeof(double));
      }

    i = 0;
    for(inc=0; inc<lim_th; inc++) {
    if(Index_result[inc] == 1){
	for(j=0; j<dim_th; j++) {
	  Theta_result[i][j] = Theta_mat[inc][j];
	  /*printf("%.2f ",Theta_mat[inc][j]);*/
	}
      i++;
      }
    }
    
        /*   for (i=0;i<length_result;i++){
	for (j=0;j<dim_th;j++){
		printf("%.2f ",Theta_result[i][j]);}
	  printf("\n");
          }*/

  printf("Start test of %lu games\n",nbjeu);

/* ---------------Testing coverage of the identified set-------------*/
  #pragma omp parallel for default(shared) private(jnc,inc,fail_m,i,j,m,row,L,t,test_stop,Q,alpha,beta,c,th,g1,g2) reduction(+:rejection)
  for (jnc=0;jnc<nbjeu;jnc++){
    
    Q= malloc(dim_eql_eg*sizeof(double));
    for(i=0; i<dim_eql_eg; i++) {
      Q[i] = 0;
      }
    inc=0;
    do {
      /*if((inc ==0)||(inc == length_result-1)){*/
      fail_m = 0;
      	
      alpha = Theta_result[inc][0];
      beta  = Theta_result[inc][1];
      c     = alpha;
      th    = Theta_result[inc][2];
      /*printf("%f,%f,%f\n",alpha,c,th);	*/
      m = 0;    
      test_stop = 0;
    
      do { /*market iteration*/
      	t = 0.0;
        g1  = xplic_mat[m][0];
	g2  = xplic_mat[m][1];
        t = Q_lkhd_eg(Q,alpha,th,beta*g1,beta*g2);
        if(fabs(t-1.0)>TOL) printf("Problem! value of lkhd:%f\n",t);
	row=obs*jnc+m;
	    
	i=0;
      	do{	
	  L=0;
	  for (j=0; j<dim_eql_eg; j++){
	    L = L + adj_mat_eg[i][j]*Q[j];
	    /*printf("%f,%f,%f\n",L,adj_mat_eg[m][i],Q[j]);*/
	  }
	  /*printf("%f,%f\n",L,phat_mat_eg[row][i]);*/
	  
	  if (L - phat_mat_eg[row][i] < - TOL) test_stop++;
	  i++;
	} while ((test_stop == 0) && (i < dim_p_eg));
      	    
        if (test_stop > 0.0) fail_m++;
        m++;
      } while((m < obs) && (fail_m <= tolerance));
	
     inc++;
     /*}
     else{
     inc++;}    */  
    } while ((inc<length_result) && (fail_m <= tolerance )); /*end for loop on inc for each theta*/
    if (fail_m > tolerance) rejection++;  
    free(Q);
 }/*end for loop on jnc for each game*/
 
  /*-----------------End Open MP section-------------------*/

  result = Write_File(Theta_result,"Theta_result.bin",length_result,dim_th);
  if (result == 0) printf("Yeah!!! - Exported th_result.bin\n");
    
  printf("Proportion of rejections:%lu\n",rejection);
  /*    result = Read_File(Theta_result,"Theta_result.bin",length_result,dim_th);
        for (i=0;i<length_result;i++){
	for (j=0;j<dim_th;j++)
  {	printf("%.2f ",Theta_result[i][j]);}
	printf("\n");
  }*/

  for(i=0; i<length_result; i++) {
    free(Theta_result[i]);
    }
    free(Theta_result);
  
} /*end if(theta_result>0)*/
  
free(Index_result);

for(i=0; i<obs*nbjeu; i++) {
    free(phat_mat_eg[i]);
  }
  free(phat_mat_eg);


for(i=0; i<obs; i++) {
    free(xplic_mat[i]);
  }
  free(xplic_mat);

for(i=0; i<lim_th; i++) {
    free(Theta_mat[i]);
  }
  free(Theta_mat);

/*finalize_random_generator();*/

time(&end);

  dif = difftime(end,start);
  printf("Elapsed time: %.2f\n", dif);
  return 0;
}

/*----------------------------------------------------------------------*/
/*  Level 2       */
/*----------------------------------------------------------------------*/

double Q_lkhd_eg(double* P, double c, double th, double bg1, double bg2)
{
/*Equilibrium designation
%0:OO
%1:OL
%2:LO
%3:LL
%4:HH
%5:OO LH
%6:OL LH
%7:LL OO
%9:LL LH
%10:HL OO
%11:HL LO
%12:HL LL
%13:HH OO
%14:HH OL
%15:HH LO
%16:HH LH
%17:HH HL
%18:LH LL OO
%19:HL LL OO
%20:HH LH OL
%21:HH LH OO
%22:HH HL OO
%23:HH HL LO
%24:HH HL LH LL
%25:HH LL OO LH HL*/
double test=0;
int i;
double region[16];
double a[5];
double b[5];

for(i=0;i<dim_eql_eg;i++){
  P[i]=0.0;
  }

a[0]=b[0]=-th;
a[3]=b[3]=c;
a[4]=b[4]=th;

if ((bg1<0.0)&&(bg2<0.0)){
a[1]=b[1]=-th;
a[2]= c-c*(1.0+bg1);
b[2]= c-c*(1.0+bg2);
C_region(region,a,b);
P[0]=region[5]+region[6]+region[9];
P[2]=region[7];
P[1]=region[13];
P[3]=region[15]+region[14]+region[11];
P[7]=region[10];}
else{
if ((bg1>0.0)&&(bg2>0.0)){
a[1]= 2.0*c - 3.0*c*(1.0+bg1);
b[1]= 2.0*c - 3.0*c*(1.0+bg2);
a[2]= 2.0*(c-c*(1.0+bg1));
b[2]= 2.0*(c-c*(1.0+bg2));
C_region(region,a,b);
P[0]=region[0]+region[1]+region[2]+region[4]+region[8];
P[1]=region[12];
P[2]=region[3];
P[4]=region[15]+region[14]+region[11];
P[14]=region[13];
P[15]=region[7];
P[13]=region[5]+region[6]+region[9]+region[10];}
else{
if ((bg1==0.0)&&(bg2==0.0)){
a[1] = b[1] =-c;
a[2] = b[2] = 0.0;
C_region(region,a,b);
P[0]=region[0]+region[1]+region[2]+region[4]+region[8];
P[1]=region[12];
P[2]=region[3];
P[13]=region[5];
P[20]=region[13];
P[21]=region[9];
P[22]=region[6];
P[23]=region[7];
P[24]=region[15]+region[14]+region[11];
P[25]=region[10];
}
else{
if ((bg1<0.0)&&(bg2==0.0)){
a[1] = c - 2.0*c*(1.0+bg1);
a[2] = c - c*(1.0+bg1);
b[1] = -c;
b[2] = 0.0;
C_region(region,a,b);
P[0]=region[0]+region[1]+region[2]+region[4]+region[5]+region[6]+region[8];
P[1]=region[12];
P[2]=region[3]+region[7];
P[6]=region[13];
P[5]=region[9];
P[9]=region[15]+region[14]+region[11];
P[18]=region[10];
}
else{
if ((bg1==0.0)&&(bg2<0.0)){
b[1] = c - 2.0*c*(1.0+bg1);
b[2] = c - c*(1.0+bg1);
a[1] = -c;
a[2] = 0.0;
C_region(region,a,b);
P[0]=region[0]+region[1]+region[2]+region[4]+region[5]+region[8]+region[9];
P[1]=region[12]+region[13];
P[2]=region[3];
P[10]=region[6];
P[11]=region[7];
P[12]=region[15]+region[14]+region[11];
P[19]=region[10];
}
else{
if ((bg1>0.0)&&(bg2==0.0)){
a[1] = 2.0*c - 3.0*c*(1.0+bg1);
a[2] = 2.0*(c - c*(1.0+bg1));
b[1] = -c;
b[2] = 0.0;
C_region(region,a,b);
P[0]=region[0]+region[1]+region[2]+region[4]+region[8];
P[1]=region[12];
P[2]=region[3];
P[13]=region[5]+region[9];
P[14]=region[13];
P[22]=region[6]+region[10];
P[23]=region[7];
P[17]=region[15]+region[14]+region[11];
}
else{
if ((bg1==0.0)&&(bg2>0.0)){
b[1] = 2.0*c - 3.0*c*(1.0+bg1);
b[2] = 2.0*(c - c*(1.0+bg1));
a[1] = -c;
a[2] = 0.0;
C_region(region,a,b);
P[0]=region[0]+region[1]+region[2]+region[4]+region[8];
P[1]=region[12];
P[2]=region[3];
P[13]=region[5]+region[6];
P[20]=region[13];
P[21]=region[9]+region[10];
P[15]=region[7];
P[16]=region[15]+region[14]+region[11];
}}}}}}}

for (i=0;i<dim_eql_eg;i++){
  test = test+P[i];
}
return test;
}

/*----------------------------------------------------------------------*/
/*  Level 3       */
/*----------------------------------------------------------------------*/

void C_region(double* region, double* a,double* b)
{
double l[4],L[4];
int i,j,row;
for(i=0;i<4;i++){
  l[i] = a[i+1] - a[i];
  L[i] = b[i+1] - b[i];
  }
  
for (i=0;i<4;i++){
  for(j=0;j<4;j++){
  row = (int)4*i+j;
  region[row] = (l[i]*L[j])/((a[4]-a[0])*(b[4]-b[0]));
  /*printf("%f\n",region[row]);*/
  }
 }

}





  
