%% Monte Carlo exercise in Henry, Mango, Queyranne (2014):
% Selection mechanism: Maximum
%% Initialization

nb.jeu  = 5000;
nb.obs  = 100;
nb.bs   = 999;
%nb.conf_level = 0.99;
nb.multinomial = 0;
nb.smooth=0;
nb.option = 7;
%nb.eql = 10;
nb.r_x = nb.obs;
nb.dim_x = 2;

alpha_0 = 0.0;
c_0 = 0.0;
beta_0 = 0.0;
th_0 = 0.5;

%% Identified set for beta = 0 and maximal selection;

%OO
P(1) = (-c_0 - (-th_0))*(c_0 - (-th_0)) + 2*c_0*(-c_0 -(-th_0));
%OL
P(2) = (-c_0 -(-th_0))*(th_0 - c_0);
%LO
P(3) = P(2);
%LL
P(4) = 0;
%HL
P(5) = 0;
%LH
P(6) = 0;
%HH
P(7) = ((th_0 - (-c_0))^2);

P_unif = P(ones(2^nb.dim_x,1),:)/((2*th_0)^2);

%% All possible combination of equilibrium

E = x_grid(eye(7));
E_rev = zeros(size(E));
for i=1:length(E)
    E_rev(i,:) = E(length(E) - i +1,:);
end

P_ident = (E_rev*P_unif')';
Adj_eg  = adjacency_dec(E);

% Define target for the results
%binifile(P_ident, 'C:\Users\Meango\Documents\MATLAB\simulation QE\revision\results\maximal\00\100\pident_mat_eg.bin','double')

%% Simulation d'un jeu g avec "maximal selection"
tic
P_eg = zeros(nb.jeu,2^nb.option);
P_eg_99 = zeros(2*nb.jeu,2^nb.option);
P_eg_95 = zeros(2*nb.jeu,2^nb.option);
P_eg_90 = zeros(2*nb.jeu,2^nb.option);
grid = [0 0;0 1;1 0;1 1];
rand('seed',654321)
for g = 1:nb.jeu
    
    % Donnees du probleme
    G = round(rand(nb.obs,nb.dim_x));  
    % Find all the possible combination of X.
    %grid=x_grid(G);
    % Find the vectors that are the same and define their neighbors.
    x_eq=x_refd(G,grid);
    % lgrid will be used for the estimation
    nb.lgrid=length(grid(:,1));
    
    Epsi = (rand(nb.obs,2)-0.5)*2*th_0;
    Y = zeros(nb.obs,1);


    if alpha_0 == c_0
        for i = 1:nb.obs
            epsi = Epsi(i,:);

            %OL
            if (epsi(1) < -c_0) && (epsi(2)>c_0)
                Y(i) =2;
                %HH
            elseif (epsi(1) > - c_0) && (epsi(2) > - c_0)
                Y(i) = 7;

                %LO
            elseif (epsi(1) > c_0) && (epsi(2)<-c_0)
                Y(i) =3;
                %OO
            else Y(i) = 1;
            end
        end
    end

    [PHAT,Gn,W_b] = bootstrap_sim_x(Y,nb.bs,nb,x_eq);
    PHAT_bar99=dec_PHAT_sim_x(PHAT,Gn,W_b,0.99,x_eq,nb.lgrid);
    PHAT_bar95=dec_PHAT_sim_x(PHAT,Gn,W_b,0.95,x_eq,nb.lgrid);
    PHAT_bar90=dec_PHAT_sim_x(PHAT,Gn,W_b,0.90,x_eq,nb.lgrid);

    row_inf = 2^nb.dim_x*(g-1)+1;
    row_sup = row_inf+(2^nb.dim_x-1);
    P_eg_99(row_inf:row_sup,:) = (E_rev*PHAT_bar99')';
    P_eg_95(row_inf:row_sup,:) = (E_rev*PHAT_bar95')';
    P_eg_90(row_inf:row_sup,:) = (E_rev*PHAT_bar90')';
    
    if g/1000 == round(g/1000)
        disp(sprintf('Game iteration: %f' ,g))
    end
end
toc

% Define target for the "C folder"
%binifile(P_eg_99, 'C:\Users\Meango\Documents\MATLAB\simulation QE\revision\results\maximal\00\100\phat_mat_eg_99_ver.bin','double')
%binifile(P_eg_95, 'C:\Users\Meango\Documents\MATLAB\simulation QE\revision\results\maximal\00\100\phat_mat_eg_95_ver.bin','double')
%binifile(P_eg_90, 'C:\Users\Meango\Documents\MATLAB\simulation QE\revision\results\maximal\00\100\phat_mat_eg_90_ver.bin','double')
%binifile(grid, 'C:\cygwin\home\MEANGO\care_sim\Xplic.bin','double')

%% Monte carlo for PHAT
rejection(1) = 0;
for g=1:nb.jeu
    inc=0;
    i=1;
    while ((inc ==0) && (i<=2^nb.dim_x))
        P_eg=P_eg_99(2^nb.dim_x*(g-1)+i,:);
        A=sum((P_eg<=P_ident(1,:)),2);
        if A<128
            inc=1;
        end
        i=i+1;
    end
    rejection(1) = rejection(1)+inc;
end

rejection(2) = 0;
for g=1:nb.jeu
    inc=0;
    i=1;
    while ((inc ==0) && (i<=2^nb.dim_x))
        P_eg=P_eg_95(2^nb.dim_x*(g-1)+i,:);
        A=sum((P_eg<=P_ident(1,:)),2);
        if A<128
            inc=1;
        end
        i=i+1;
    end
    rejection(2) = rejection(2)+inc;
end

rejection(3) = 0;
for g=1:nb.jeu
    inc=0;
    i=1;
    while ((inc ==0) && (i<=2^nb.dim_x))
        P_eg=P_eg_90(2^nb.dim_x*(g-1)+i,:);
        A=sum((P_eg<=P_ident(1,:)),2);
        if A<128
            inc=1;
        end
        i=i+1;
    end
    rejection(3) = rejection(3)+inc;
end

% Define target for the results
%binifile(rejection, 'C:\Users\Meango\Documents\MATLAB\simulation QE\revision\results\maximal\00\100\rejection_ver.bin','double')

